@@ -28,6 +28,7 @@
 from __future__ import unicode_literals
 
 import os
+import re
 import logging
 import logging.handlers
 import base64
@@ -38,9 +39,19 @@ import poster
 import time
 #from datetime import datetime
 import datetime
+from ordereddict import OrderedDict
 from progressbar import ProgressBar, FileTransferSpeed, Bar, ETA, Percentage
 import hashlib
 
+# -----------------------------------------------------------------------------
+class LinShareException(Exception):
+    pass
+
+#    def __init__(self, code, msg, *args, **kwargs):
+#        super(LinShareException, self).__init__(*args, **kwargs)
+#        self.code = code
+#        self.msg = msg
+
 # -----------------------------------------------------------------------------
 def extract_file_name(content_dispo):
     """Extract file name from the input request body"""
@@ -53,7 +64,7 @@ def extract_file_name(content_dispo):
     for key_val in content_dispo.split(';'):
         param = key_val.strip().split('=')
         if param[0] == "filename":
-            file_name = param[1]
+            file_name = param[1].strip('"')
             break
     return file_name
 
@@ -202,7 +213,7 @@ class CoreCli(object):
         if self.debug:
             try:
                 self.debuglevel = int(self.debug)
-            except ValueError as e:
+            except ValueError:
                 self.debuglevel = 1
 
             if self.debuglevel >= 3:
@@ -218,7 +229,7 @@ class CoreCli(object):
                 uri=host.encode('utf8'),
                 user=user.encode('utf8'),
                 passwd=password.encode('utf8'))
-        except UnicodeEncodeError as ex:
+        except UnicodeEncodeError:
             self.log.error("the program was not able to compute the basic authentication token.")
 
         handlers = [
@@ -267,7 +278,6 @@ class CoreCli(object):
         request.add_header("Authorization", "Basic %s" % base64string)
         #request.add_header("Cookie", "JSESSIONID=")
 
-
     def get_json_result(self, resultq):
         jObj  = None
         result = resultq.read()
@@ -286,10 +296,51 @@ class CoreCli(object):
             raise ValueError(msg)
         return jObj
 
+    def do_request(self, request):
+        # request start
+        self.last_req_time = None
+        jObj = None
+        starttime = datetime.datetime.now()
+        try:
+            # doRequest
+            resultq = urllib2.urlopen(request)
+            code = resultq.getcode()
+
+            if self.verbose or self.debug:
+                self.log.info("http return code : " + str(code))
+
+            if code == 200:
+                jObj = self.get_json_result(resultq)
+
+        except urllib2.HTTPError as ex:
+            code = "-1"
+            msg = "Http error : " + ex.msg + " (" + str(ex.code) + ")"
+            if self.verbose:
+                self.log.info(msg)
+            else:
+                self.log.debug(msg)
+
+            if ex.code != 405:
+                jObj = self.get_json_result(ex)
+                code = jObj.get('errCode')
+                msg = jObj.get('message')
+                self.log.debug("Server error code : " + str(code))
+                self.log.debug("Server error message : " + str(msg))
+
+            # request end
+            endtime = datetime.datetime.now()
+            self.last_req_time = str(endtime - starttime)
+            raise LinShareException(code, msg)
+
+        # request end
+        endtime = datetime.datetime.now()
+
+        self.last_req_time = str(endtime - starttime)
+        return jObj
+
     @cli_get_cache
     def list(self, url):
-        """ List all documents store into LinShare."""
-        self.last_req_time = None
+        """ List ressources store into LinShare."""
         url = self.root_url + url
         self.log.debug("list url : " + url)
 
@@ -298,115 +349,90 @@ class CoreCli(object):
         request.add_header('Content-Type', 'application/json; charset=UTF-8')
         request.add_header('Accept', 'application/json')
 
-        # request start
-        starttime = datetime.datetime.now()
-        jObj = None
-        try:
-            # doRequest
-            resultq = urllib2.urlopen(request)
-            code = resultq.getcode()
-            if code == 204:
-                self.log.info("ret code : " + str(code))
-            elif code == 200:
-                self.log.info("ret code : " + str(code))
-                jObj = self.get_json_result(resultq)
-            else:
-                self.log.error("ret code : " + str(code))
-        except urllib2.HTTPError as e:
-            self.log.error("Http error : " + e.msg)
-            self.log.error("error code : " + str(e.code))
+        ret = self.do_request(request)
 
-        # request end
-        endtime = datetime.datetime.now()
-
-        self.last_req_time = str(endtime - starttime)
         self.log.debug("""list url : %(url)s : request time : %(time)s""",
                        {"url": url,
                         "time": self.last_req_time})
-        return jObj
+        return ret
 
-    def delete(self, url, data = None):
-        """ List all documents store into LinShare."""
-        self.last_req_time = None
+    def delete(self, url, data=None):
+        """Delete one ressource store into LinShare."""
         url = self.root_url + url
-        self.log.debug("list url : " + url)
+        self.log.debug("delete url : " + url)
 
         # Building request
         request = urllib2.Request(url)
         if data:
             # Building request
-            headers = {'content-type': 'application/json'}
-            #request.add_header('Content-Type:', 'application/json; charset=UTF-8')
             post_data = json.dumps(data).encode("UTF-8")
-            request = urllib2.Request(url, post_data, headers=headers)
+            request = urllib2.Request(url, post_data)
+            request.add_header('Content-Type', 'application/json; charset=UTF-8')
+            request.add_header('Accept', 'application/json')
 
         request.get_method = lambda: 'DELETE'
 
-        # request start
-        starttime = datetime.datetime.now()
-        jObj = None
-        try:
-            # doRequest
-            resultq = urllib2.urlopen(request)
-            code = resultq.getcode()
-            if code == 204:
-                self.log.info("ret code : " + str(code))
-            elif code == 200:
-                self.log.info("ret code : " + str(code))
-                jObj = self.get_json_result(resultq)
-            else:
-                self.log.error("ret code : " + str(code))
-        except urllib2.HTTPError as e:
-            self.log.error("Http error : " + e.msg)
-            self.log.error("error code : " + str(e.code))
+        ret = self.do_request(request)
 
-        # request end
-        endtime = datetime.datetime.now()
-
-        self.last_req_time = str(endtime - starttime)
         self.log.debug("""delete url : %(url)s : request time : %(time)s""",
                        {"url": url,
                         "time": self.last_req_time})
-        return jObj
+        return ret
 
-    def create(self, url, data):
-        """ List all documents store into LinShare."""
-        self.last_req_time = None
+    def options(self, url):
         url = self.root_url + url
-        self.log.debug("list url : " + url)
+        self.log.debug("options url : " + url)
+
+        # Building request
+        request = urllib2.Request(url)
+        request.add_header('Content-Type', 'application/json; charset=UTF-8')
+        request.add_header('Accept', 'application/json')
+
+        request.get_method = lambda: 'OPTIONS'
+
+        ret = self.do_request(request)
+
+        self.log.debug("""options url : %(url)s : request time : %(time)s""",
+                       {"url": url,
+                        "time": self.last_req_time})
+        return ret
+
+    def create(self, url, data):
+        """ create ressources store into LinShare."""
+        url = self.root_url + url
+        self.log.debug("create url : " + url)
 
         # Building request
-        headers = {'content-type': 'application/json'}
-        #request.add_header('Content-Type:', 'application/json; charset=UTF-8')
         post_data = json.dumps(data).encode("UTF-8")
-        request = urllib2.Request(url, post_data, headers=headers)
+        request = urllib2.Request(url, post_data)
+        request.add_header('Content-Type', 'application/json; charset=UTF-8')
+        request.add_header('Accept', 'application/json')
 
-        # request start
-        starttime = datetime.datetime.now()
+        ret = self.do_request(request)
 
-        jObj = None
-        try:
-            # doRequest
-            resultq = urllib2.urlopen(request)
-            code = resultq.getcode()
-            if code == 204:
-                self.log.info("ret code : " + str(code))
-            elif code == 200:
-                self.log.info("ret code : " + str(code))
-            else:
-                self.log.error("ret code : " + str(code))
-        except urllib2.HTTPError as e:
-            self.log.error("Http error : " + e.msg)
-            self.log.error("error code : " + str(e.code))
-
-        # request end
-        endtime = datetime.datetime.now()
-
-        self.last_req_time = str(endtime - starttime)
         self.log.debug("""post url : %(url)s : request time : %(time)s""",
                        {"url": url,
                         "time": self.last_req_time})
-        return jObj
+        return ret
+
+    def update(self, url, data):
+        """ update ressources store into LinShare."""
+        url = self.root_url + url
+        self.log.debug("update url : " + url)
+
+        # Building request
+        post_data = json.dumps(data).encode("UTF-8")
+        request = urllib2.Request(url, post_data)
+        request.add_header('Content-Type', 'application/json; charset=UTF-8')
+        request.add_header('Accept', 'application/json')
+        request.get_method = lambda: 'PUT'
+
+        ret = self.do_request(request)
+
+        self.log.debug("""put url : %(url)s : request time : %(time)s""",
+                       {"url": url,
+                        "time": self.last_req_time})
+        return ret
 
     def upload(self, file_path, url, description=None):
         self.last_req_time = None
@@ -421,10 +447,9 @@ class CoreCli(object):
         self.log.debug("file_name is : " + file_name)
 
         if file_size <= 0:
-            self.log.fatal("""The file '%(filename)s' can not be uploaded
-because its size less or equal to zero.""",
-                           {"filename": str(file_name)})
-            return None
+            msg = "The file '%(filename)s' can not be uploaded \
+because its size is less or equal to zero." % {"filename": str(file_name)}
+            raise LinShareException("-1", msg)
 
         widgets = [FileTransferSpeed(), ' <<<', Bar(), '>>> ',
                    Percentage(), ' ', ETA()]
@@ -442,6 +467,7 @@ because its size less or equal to zero.""",
 
         # Building request
         request = urllib2.Request(url, datagen, headers)
+        request.add_header('Accept', 'application/json')
 
         # request start
         pbar.start()
@@ -451,41 +477,52 @@ because its size less or equal to zero.""",
         try:
             # doRequest
             resultq = urllib2.urlopen(request)
-        except urllib2.HTTPError as e:
-            self.log.error("Http error : " + e.msg)
-            self.log.error("error code : " + str(e.code))
+            code = resultq.getcode()
+
+            if self.verbose or self.debug:
+                self.log.info("http return code : " + str(code))
+
+            if code == 200:
+                jObj = self.get_json_result(resultq)
+
+        except urllib2.HTTPError as ex:
+
+            if self.verbose:
+                self.log.info("Http error : " + ex.msg + " (" + str(ex.code) + ")")
+            else:
+                self.log.debug("Http error : " + ex.msg + " (" + str(ex.code) + ")")
+            jObj = self.get_json_result(ex)
+            code = jObj.get('errCode')
+            msg = jObj.get('message')
+            self.log.debug("Server error code : " + str(code))
+            self.log.debug("Server error message : " + str(msg))
 
             # request end
             endtime = datetime.datetime.now()
             pbar.finish()
-
             self.last_req_time = str(endtime - starttime)
-            if e.code == 500:
-                self.log.fatal(
+
+            if ex.code == 502:
+                self.log.warn(
+                    """The file '%(filename)s' was uploaded
+                    (%(time)ss) but the proxy cut the connexion. No server
+                    acknowledge was received.""",
+                    {"filename": file_name,
+                     "time": self.last_req_time})
+            else:
+                self.log.debug(
                     "Can not upload file %(filename)s (%(filepath)s)",
                     {"filename": file_name,
                      "filepath": file_path})
-            else:
-                self.log.error(
-                    """The file '%(filename)s' was uploaded
-                    (%(time)ss) but the proxy cut the connexion. No server
-                    acquitment was received.""",
-                    {"filename": file_name,
-                     "time": self.last_req_time})
-            return None
+            raise LinShareException(code, msg)
+
 
         # request end
         endtime = datetime.datetime.now()
         pbar.finish()
-        result = resultq.read()
-
-        jObj = json.loads(result)
-        if self.debuglevel >= 2:
-            self.log.debug("the result is : ")
-            self.log.debug(json.dumps(jObj, sort_keys=True, indent=2))
 
         self.last_req_time = str(endtime - starttime)
-        self.log.debug("list url : %(url)s : request time : %(time)s",
+        self.log.debug("upload url : %(url)s : request time : %(time)s",
                        {"url": url,
                         "time": self.last_req_time})
         return jObj
@@ -499,6 +536,8 @@ This method could throw exceptions like urllib2.HTTPError."""
 
         # Building request
         request = urllib2.Request(url)
+        #request.add_header('Content-Type', 'application/json; charset=UTF-8')
+        request.add_header('Accept', 'application/json;charset=UTF-8')
 
         # request start
         starttime = datetime.datetime.now()
@@ -512,7 +551,7 @@ This method could throw exceptions like urllib2.HTTPError."""
         if code == 200:
             content_lenth = resultq.info().getheader('Content-Length')
             if not content_lenth:
-                print "ERRRRRRRRRRRRRRRRRRROOOOOOOOOOOOORR"
+                self.log.error("No content lengh header found !")
                 result = resultq.read()
                 print result
                 return
@@ -520,7 +559,6 @@ This method could throw exceptions like urllib2.HTTPError."""
             content_dispo = resultq.info().getheader('Content-disposition')
             content_dispo = content_dispo.strip()
             file_name = extract_file_name(content_dispo)
-
             if os.path.isfile(file_name):
                 cpt = 1
                 while 1:
@@ -553,12 +591,108 @@ This method could throw exceptions like urllib2.HTTPError."""
         # request end
         endtime = datetime.datetime.now()
         self.last_req_time = str(endtime - starttime)
-        self.log.debug("list url : %(url)s : request time : %(time)s",
+        self.log.debug("download url : %(url)s : request time : %(time)s",
                        {"url": url,
                         "time": self.last_req_time})
         return (file_name, self.last_req_time)
 
 
+class ResourceBuilder(object):
+
+    def __init__(self, name=None, required=False):
+        self._name = name
+        self._fields = OrderedDict()
+        self._required = required
+
+    def add_field(self, field, arg=None, value=None, extended=False, hidden=False, e_type=str, required=None):
+        if required is None:
+            required = self._required
+        if arg is None:
+            arg = re.sub('(?!^)([A-Z]+)', r'_\1', field).lower()
+        self._fields[field] = {
+            'field': field,
+            'arg': arg,
+            'value': value,
+            'extended': extended,
+            'required': required,
+            'e_type': e_type,
+            'hidden': hidden
+        }
+
+    def get_keys(self, extended=False):
+        res = []
+        for field in self._fields.values():
+            if field['hidden']:
+                continue
+            if not field['extended']:
+                res.append(field['field'])
+            if extended and field['extended']:
+                res.append(field['field'])
+        return res
+
+    def get_fields(self, extended=False, full=False):
+        res = []
+        if extended:
+            for field in self._fields.values():
+                if field['extended']:
+                    res.append(field['field'])
+        elif full:
+            for field in self._fields.keys():
+                res.append(field)
+        else:
+            for field in self._fields.values():
+                if not field['extended']:
+                    res.append(field['field'])
+        return res
+
+    def set_arg(self, key, arg):
+        field = self._fields.get(key, None)
+        if field is not None:
+            field['arg'] = arg
+
+    def set_value(self, key, value):
+        field = self._fields.get(key, None)
+        if field is not None:
+            field['value'] = value
+
+    def to_resource(self):
+        ret = {}
+        for field in self._fields.values():
+            ret[field['field']] = field['value']
+        return ret
+
+    def load_from_args(self, namespace):
+        for field in self._fields.values():
+            value = getattr(namespace, field['arg'], None)
+            if value is not None:
+                field['value'] = value
+
+    def copy(self, data):
+        if isinstance(data, dict):
+            for field, val in self._fields.items():
+                val['value'] = data.get(field, "")
+
+        if isinstance(data, ResourceBuilder):
+            for field, val in self._fields.items():
+                val['value'] = data[field]['value']
+
+    def __str__(self):
+        return json.dumps(self.to_resource(), sort_keys=True, indent=2)
+
+    def check_required_fields(self):
+        for field in self._fields.values():
+            if field['required']:
+                value = field['value']
+                if value is None:
+                    raise ValueError("missing value for required field : "
+                                     + field['field'])
+                e_type = field['e_type']
+                if e_type == int:
+                    int(value)
+                if e_type == float:
+                    float(value)
+
+
 # -----------------------------------------------------------------------------
 # USER API
 # -----------------------------------------------------------------------------
@@ -568,19 +702,19 @@ class Documents(object):
 
     def list(self):
         """ List all documents store into LinShare."""
-        return self.core.list("documents.json")
+        return self.core.list("documents")
 
     def upload(self, file_path, description=None):
         """ Upload a file to LinShare using its rest api.
         The uploaded document uuid will be returned"""
-        return self.core.upload(file_path, "documents.json", description)
+        return self.core.upload(file_path, "documents", description)
 
     def download(self, uuid):
         url = "documents/%s/download" % uuid
         return self.core.download(uuid, url)
 
     def delete(self, uuid):
-        url = "documents/%s.json" % uuid
+        url = "documents/%s" % uuid
         return self.core.delete(url)
 
 
@@ -591,7 +725,7 @@ class ReceivedShares(object):
         self.core = corecli
 
     def list(self):
-        return self.core.list("shares.json")
+        return self.core.list("shares")
 
     def download(self, uuid):
         url = "shares/%s/download" % uuid
@@ -644,7 +778,7 @@ class Threads(object):
         self.core = corecli
 
     def list(self):
-        return self.core.list("threads.json")
+        return self.core.list("threads")
 
 
 # -----------------------------------------------------------------------------
@@ -653,7 +787,7 @@ class ThreadsMembers(object):
         self.core = corecli
 
     def list(self, threadUuid):
-        url = "thread_members/%s.json" % threadUuid
+        url = "thread_members/%s" % threadUuid
         return self.core.list(url)
 
 
@@ -663,7 +797,7 @@ class Users(object):
         self.core = corecli
 
     def list(self):
-        return self.core.list("users.json")
+        return self.core.list("users")
 
 
 # -----------------------------------------------------------------------------
@@ -681,68 +815,174 @@ class UserCli(CoreCli):
 # -----------------------------------------------------------------------------
 # ADMIN API
 # -----------------------------------------------------------------------------
-class DomainAdmins(object):
+class GenericAdminClass(object):
     def __init__(self, corecli):
         self.core = corecli
+        self.log = logging.getLogger('linshare-cli.rbu')
+
+    def get_rbu(self):
+        rbu = ResourceBuilder("generic")
+        return rbu
+
+    def get_resource(self):
+        return self.get_rbu().to_resource()
+
+    def debug(self, data):
+        self.log.debug("input data :")
+        self.log.debug(json.dumps(data, sort_keys=True, indent=2))
+
+    def check(self, data):
+        rbu = self.get_rbu()
+        rbu.copy(data)
+        rbu.check_required_fields()
+
+
+class DomainAdmins(GenericAdminClass):
 
     def list(self):
-        return self.core.list("admin/domains.json")
-
-
-class DomainPatternsAdmin(object):
-    def __init__(self, corecli):
-        self.core = corecli
-
-    def list(self, model = False):
-        if model:
-            return self.core.list("admin/domain_patterns/models.json")
-        else:
-            return self.core.list("admin/domain_patterns")
-            return self.core.list("admin/domain_patterns.json")
+        return self.core.list("admin/domains")
 
     def create(self, data):
-        return self.core.create("admin/domain_patterns.json", data)
+        self.debug(data)
+        self.check(data)
+        return self.core.create("admin/domains", data)
 
-    def delete(self, id_dp):
-        data = { "identifier":  id_dp}
-        data = id_dp
-        return self.core.delete("admin/domain_patterns.json", data)
+    def update(self, data):
+        self.debug(data)
+        return self.core.update("admin/domains", data)
+
+    def delete(self, identifier):
+        if identifier:
+            identifier = identifier.strip(" ")
+        if not identifier:
+            raise ValueError("identifier is required")
+        data = {"identifier":  identifier}
+        return self.core.delete("admin/domains", data)
+
+    def options_language(self):
+        return self.core.options("admin/enums/language")
+
+    def options_role(self):
+        my_list = self.core.options("admin/enums/role")
+        return filter(lambda x: x not in ["SUPERADMIN", "SYSTEM"], my_list)
+
+    def options_type(self):
+        my_list = self.core.options("admin/enums/domain_type")
+        return filter(lambda x: x != "ROOTDOMAIN", my_list)
 
 
-class LdapConnectionsAdmin(object):
-    def __init__(self, corecli):
-        self.core = corecli
+    def get_rbu(self):
+        rbu = ResourceBuilder("domains")
+        rbu.add_field('identifier', required=True)
+        rbu.add_field('label', required=True)
+        rbu.add_field('policy', value={"identifier": "DefaultDomainPolicy"}, hidden=True)
+        rbu.add_field('type', "domain_type", value="TOPDOMAIN")
+        rbu.add_field('parent', "parent_id", extended=True)
+        rbu.add_field('language', value="ENGLISH")
+        rbu.add_field('userRole', "role", value="SIMPLE")
+        rbu.add_field('mailConfigUuid', value="946b190d-4c95-485f-bfe6-d288a2de1edd", extended=True)
+        rbu.add_field('mimePolicyUuid', value="3d6d8800-e0f7-11e3-8ec0-080027c0eef0", extended=True)
+        rbu.add_field('description', value="")
+        rbu.add_field('authShowOrder', value="1", extended=True)
+        rbu.add_field('providers', value=[], extended=True)
+        return rbu
+
+
+class DomainPatternsAdmin(GenericAdminClass):
+
+    def list(self, model=False):
+        if model:
+            return self.core.list("admin/domain_patterns/models")
+        else:
+            return self.core.list("admin/domain_patterns")
+
+    def create(self, data):
+        self.debug(data)
+        self.check(data)
+        return self.core.create("admin/domain_patterns", data)
+
+    def update(self, data):
+        self.debug(data)
+        return self.core.update("admin/domain_patterns", data)
+
+    def delete(self, identifier):
+        if identifier:
+            identifier = identifier.strip(" ")
+        if not identifier:
+            raise ValueError("identifier is required")
+        data = {"identifier":  identifier}
+        return self.core.delete("admin/domain_patterns", data)
+
+    def get_rbu(self):
+        rbu = ResourceBuilder("domain_patterns", required=True)
+        rbu.add_field('identifier')
+        rbu.add_field('description', value="")
+        rbu.add_field('userFirstName', 'first_name', extended=True)
+        rbu.add_field('userLastName', 'last_name', extended=True)
+        rbu.add_field('userMail', 'mail', extended=True)
+        rbu.add_field('ldapUid', extended=True)
+        rbu.add_field("authCommand", extended=True)
+        rbu.add_field("searchUserCommand", extended=True)
+        rbu.add_field("autoCompleteCommandOnAllAttributes", extended=True)
+        rbu.add_field("autoCompleteCommandOnFirstAndLastName", extended=True)
+        rbu.add_field('completionPageSize', extended=True, e_type=int)
+        rbu.add_field('completionSizeLimit', extended=True, e_type=int)
+        rbu.add_field('searchPageSize', extended=True, e_type=int)
+        rbu.add_field('searchSizeLimit', extended=True, e_type=int)
+        return rbu
+
+
+class LdapConnectionsAdmin(GenericAdminClass):
 
     def list(self):
-        return self.core.list("admin/ldap_connections.json")
+        return self.core.list("admin/ldap_connections")
+
+    def create(self, data):
+        self.debug(data)
+        self.check(data)
+        return self.core.create("admin/ldap_connections", data)
+
+    def update(self, data):
+        self.debug(data)
+        return self.core.update("admin/ldap_connections", data)
+
+    def delete(self, identifier):
+        if identifier:
+            identifier = identifier.strip(" ")
+        if not identifier:
+            raise ValueError("identifier is required")
+        data = {"identifier":  identifier}
+        return self.core.delete("admin/ldap_connections", data)
+
+    def get_rbu(self):
+        rbu = ResourceBuilder("ldap_connection")
+        rbu.add_field('identifier', required=True)
+        rbu.add_field('providerUrl', required=True)
+        rbu.add_field('securityPrincipal', "principal")
+        rbu.add_field('securityCredentials', "credential")
+        return rbu
 
 
 # -----------------------------------------------------------------------------
-class ThreadsAdmin(object):
-    def __init__(self, corecli):
-        self.core = corecli
+class ThreadsAdmin(GenericAdminClass):
 
     def list(self):
-        return self.core.list("admin/threads.json")
+        return self.core.list("admin/threads")
 
 
 # -----------------------------------------------------------------------------
-class ThreadsMembersAdmin(object):
-    def __init__(self, corecli):
-        self.core = corecli
+class ThreadsMembersAdmin(GenericAdminClass):
 
     def list(self, threadUuid):
-        url = "admin/thread_members/%s.json" % threadUuid
+        url = "admin/thread_members/%s" % threadUuid
         return self.core.list(url)
 
 
 # -----------------------------------------------------------------------------
-class UsersAdmin(object):
-    def __init__(self, corecli):
-        self.core = corecli
+class UsersAdmin(GenericAdminClass):
 
     def list(self):
-        return self.core.list("users.json")
+        return self.core.list("users")
 
 
 # -----------------------------------------------------------------------------
